"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveLibraries = exports.generateAll = exports.generate = void 0;
// Depends on fs.realpath, inflight, inherits, minimatch, once, path-is-absolute
const glob_1 = require("glob");
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const parser_1 = require("@gi.ts/parser");
function readGir(path) {
    const data = fs.readFileSync(path, {
        encoding: 'utf-8'
    });
    return parser_1.parser.parseGir(data);
}
function process_namespace(namespace) {
    var _a, _b;
    // adapt the xml2js api
    namespace.attributes = namespace[0]['$'];
    return {
        name: namespace.attributes['name'],
        api_version: namespace.attributes['version'],
        c_prefix: (_b = (_a = namespace.attributes['c:symbol-prefixes']) === null || _a === void 0 ? void 0 : _a.split(',')) !== null && _b !== void 0 ? _b : []
    };
}
function generate_slug(scraper_info) {
    return `${scraper_info.name}${scraper_info.api_version}`
        .toLowerCase()
        .trim()
        .replace(/[^\w-]/g, '');
}
function scraper_code(info) {
    return `${info.name}-${info.api_version}.gir`;
}
async function generate(gir_path, verbose = false) {
    if (typeof gir_path === 'object')
        return gir_path;
    let gir;
    try {
        if (verbose) {
            console.log(`Found ${gir_path}...`);
        }
        gir = readGir(gir_path);
    }
    catch (err) {
        console.error(err);
        console.error('Failed for ' + gir_path);
        return null;
    }
    const namespace = gir.repository[0].namespace;
    const ns_info = Object.assign({}, process_namespace(namespace));
    const scraper_info = Object.assign(Object.assign({}, ns_info), { path: gir_path, id: scraper_code(ns_info), slug: generate_slug(ns_info) });
    return scraper_info;
}
exports.generate = generate;
let force = false;
function generateAll(directories = []) {
    var _a;
    const { XDG_DATA_DIRS } = process.env;
    if (fs.existsSync('./docs-lock.json')) {
        if (force) {
            fs.unlinkSync('./docs-lock.json');
        }
        else {
            console.log('Using cached docs-lock.json');
            return JSON.parse(fs.readFileSync('./docs-lock.json').toString());
        }
    }
    const dataDirs = (_a = XDG_DATA_DIRS === null || XDG_DATA_DIRS === void 0 ? void 0 : XDG_DATA_DIRS.split(':')) !== null && _a !== void 0 ? _a : [];
    const allDirs = [...directories, ...dataDirs];
    const glob = allDirs
        .map(dir => {
        const resolved = path.resolve(dir);
        return [`${resolved}/*.gir`, `${resolved}/gir-1.0/*.gir`];
    })
        .reduce((prev, [dir, girdir]) => [...prev, ...(0, glob_1.sync)(dir), ...(0, glob_1.sync)(girdir)], []);
    return glob;
}
exports.generateAll = generateAll;
async function resolveLibraries(libraries, verbose = false, directories = []) {
    var _a;
    console.log('Finding GIR files...');
    let allLibraries = (await Promise.all(generateAll([...directories]).map(l => generate(l, verbose)))).filter((l) => l != null);
    console.log('Resolving libraries...');
    const libraryMap = new Map();
    for (const library of allLibraries) {
        const versions = libraries[library.name];
        if (!(library.name in libraries) ||
            // { Gtk: '3.0' }
            (typeof versions === 'string' && library.api_version === libraries[library.name]) ||
            // { Gtk: ['3.0', '4.0'] }
            (Array.isArray(versions) && versions.includes(library.api_version))) {
            const map = (_a = libraryMap.get(library.name)) !== null && _a !== void 0 ? _a : {};
            if (!map[library.api_version]) {
                map[library.api_version] = library;
                libraryMap.set(library.name, map);
            }
        }
    }
    const unversionedLibraries = Object.entries(libraries).map(([key]) => { var _a; return [key, (_a = libraryMap.get(key)) !== null && _a !== void 0 ? _a : null]; });
    const missingLibraries = unversionedLibraries.filter(([, l]) => l === null);
    if (missingLibraries.length > 0) {
        throw Error(`Missing Libraries!\n.The following libraries were not found: ${missingLibraries
            .map(([l]) => l)
            .join(', ')}.`);
    }
    const fullfilledLibraries = unversionedLibraries.filter((pair) => pair[1] !== null);
    const incorrectLibraries = fullfilledLibraries
        .map(([name, l]) => {
        const versions = libraries[name];
        if (typeof versions !== 'string') {
            return [name, versions.filter(v => !Object.keys(l).includes(v))];
        }
        return [name, []];
    })
        .filter(([, missingVersions]) => missingVersions.length > 0);
    for (const [name, missingVersions] of incorrectLibraries) {
        console.error(`Library ${name} is missing versions: ${missingVersions.join(', ')}.`);
    }
    if (incorrectLibraries.length > 0) {
        throw new Error(`Missing library versions!`);
    }
    return libraryMap;
}
exports.resolveLibraries = resolveLibraries;
